"use strict";
const moment = require('moment');
let models = null;
function ValidateParam(typeData, value, generatedModels, name) {
    models = generatedModels;
    if (value === undefined) {
        if (typeData.required) {
            throw new InvalidRequestException(name + ' is a required parameter.');
        }
        else {
            return undefined;
        }
    }
    switch (typeData.typeName) {
        case 'string':
            return validateString(value, name);
        case 'boolean':
            return validateBool(value, name);
        case 'number':
            return validateNumber(value, name);
        case 'array':
            return validateArray(value, typeData.arrayType, name);
        case 'datetime':
            return validateDate(value, name);
        default:
            return validateModel(value, typeData.typeName);
    }
}
exports.ValidateParam = ValidateParam;
function validateNumber(numberValue, name) {
    const parsedNumber = parseInt(numberValue, 10);
    if (isNaN(parsedNumber)) {
        throw new InvalidRequestException(name + ' should be a valid number.');
    }
    return parsedNumber;
}
function validateDate(dateValue, name) {
    const validatedDate = moment(dateValue, moment.ISO_8601, true);
    if (!validatedDate.isValid()) {
        throw new InvalidRequestException(name + ' should be a valid ISO 8601 date, i.e. YYYY-MM-DDTHH:mm:ss');
    }
    return validatedDate.toDate();
}
function validateString(stringValue, name) {
    if (typeof stringValue !== 'string') {
        throw new InvalidRequestException(name + ' should be a valid string.');
    }
    return stringValue.toString();
}
function validateBool(boolValue, name) {
    if (boolValue === true || boolValue === false) {
        return boolValue;
    }
    if (boolValue.toLowerCase() === 'true') {
        return true;
    }
    if (boolValue.toLowerCase() === 'false') {
        return false;
    }
    throw new InvalidRequestException(name + ' should be valid boolean value.');
}
function validateModel(modelValue, typeName) {
    const modelDefinition = models[typeName];
    Object.keys(modelDefinition).forEach((key) => {
        const property = modelDefinition[key];
        modelValue[key] = ValidateParam(property, modelValue[key], models, key);
    });
    return modelValue;
}
function validateArray(array, arrayType, arrayName) {
    return array.map(element => ValidateParam({
        required: true,
        typeName: arrayType,
    }, element, models, undefined));
}
class InvalidRequestException {
    constructor(message) {
        this.message = message;
        this.status = 400;
        this.name = 'Invalid Request';
    }
}
//# sourceMappingURL=templateHelpers.js.map