"use strict";
const fs = require('fs');
const mkdirp = require('mkdirp');
class SpecGenerator {
    constructor(metadata, options) {
        this.metadata = metadata;
        this.options = options;
    }
    GenerateJson(swaggerDir) {
        mkdirp(swaggerDir, dirErr => {
            if (dirErr) {
                throw dirErr;
            }
            fs.writeFile(`${swaggerDir}/swagger.json`, JSON.stringify(this.GetSpec(), null, '\t'), err => {
                if (err) {
                    throw new Error(err.toString());
                }
                ;
            });
        });
    }
    GetSpec() {
        const spec = {
            basePath: this.options.basePath,
            consumes: ['application/json'],
            definitions: this.buildDefinitions(),
            host: this.options.host,
            info: {
                description: this.options.description,
                license: {
                    name: this.options.license
                },
                title: this.options.name,
                version: this.options.version
            },
            paths: this.buildPaths(),
            produces: ['application/json'],
            swagger: '2.0'
        };
        if (this.options.host) {
            spec.host = this.options.host;
        }
        return spec;
    }
    buildDefinitions() {
        const definitions = {};
        Object.keys(this.metadata.ReferenceTypes).map(typeName => {
            const referenceType = this.metadata.ReferenceTypes[typeName];
            definitions[referenceType.name] = {
                description: referenceType.description,
                properties: this.buildProperties(referenceType.properties),
                required: referenceType.properties.filter(p => p.required).map(p => p.name),
                type: 'object'
            };
        });
        return definitions;
    }
    buildPaths() {
        const paths = {};
        this.metadata.Controllers.forEach(controller => {
            controller.methods.forEach(method => {
                const path = `${controller.path ? `/${controller.path}` : ''}${method.path}`;
                paths[path] = paths[path] || {};
                this.buildPathMethod(method, paths[path]);
            });
        });
        return paths;
    }
    buildPathMethod(method, pathObject) {
        const swaggerType = this.getSwaggerType(method.type);
        const pathMethod = pathObject[method.method] = swaggerType
            ? this.get200Operation(swaggerType, method.example, method.name)
            : this.get204Operation(method.name);
        pathMethod.description = method.description;
        pathMethod.parameters = method.parameters.map(p => this.buildParameter(p));
        if (pathMethod.parameters.filter((p) => p.in === 'body').length > 1) {
            throw new Error('Only one body parameter allowed per controller method.');
        }
    }
    buildParameter(parameter) {
        const swaggerParameter = {
            description: parameter.description,
            in: parameter.in,
            name: parameter.name,
            required: parameter.required
        };
        const parameterType = this.getSwaggerType(parameter.type);
        if (parameterType.$ref) {
            swaggerParameter.schema = parameterType;
        }
        else {
            swaggerParameter.type = parameterType.type;
        }
        return swaggerParameter;
    }
    buildProperties(properties) {
        const swaggerProperties = {};
        properties.forEach(property => {
            const swaggerType = this.getSwaggerType(property.type);
            swaggerType.description = property.description;
            swaggerProperties[property.name] = swaggerType;
        });
        return swaggerProperties;
    }
    getSwaggerType(type) {
        if (typeof type === 'string' || type instanceof String) {
            return this.getSwaggerTypeForPrimitiveType(type);
        }
        const arrayType = type;
        if (arrayType.elementType) {
            return this.getSwaggerTypeForArrayType(arrayType);
        }
        return this.getSwaggerTypeForReferenceType(type);
    }
    getSwaggerTypeForPrimitiveType(primitiveTypeName) {
        const typeMap = {};
        typeMap['number'] = { format: 'int64', type: 'integer' };
        typeMap['string'] = { type: 'string' };
        typeMap['boolean'] = { type: 'boolean' };
        typeMap['datetime'] = { format: 'date-time', type: 'string' };
        typeMap['object'] = { type: 'object' };
        typeMap['void'] = null;
        return typeMap[primitiveTypeName];
    }
    getSwaggerTypeForArrayType(arrayType) {
        const elementType = arrayType.elementType;
        return { items: this.getSwaggerType(elementType), type: 'array' };
    }
    getSwaggerTypeForReferenceType(referenceType) {
        return { $ref: `#/definitions/${referenceType.name}` };
    }
    get200Operation(swaggerType, example, methodName) {
        return {
            operationId: methodName,
            produces: ['application/json'],
            responses: {
                '200': { description: '', examples: { 'application/json': example }, schema: swaggerType }
            }
        };
    }
    get204Operation(methodName) {
        return {
            operationId: methodName,
            responses: {
                '204': { description: 'No content' }
            }
        };
    }
}
exports.SpecGenerator = SpecGenerator;
//# sourceMappingURL=specGenerator.js.map